@@ -3,8 +3,6 @@
 #include "plugins_d.h"
 
 char *plugin_directories[PLUGINSD_MAX_DIRECTORIES] = { NULL };
-char *netdata_configured_plugins_dir_base;
-
 struct plugind *pluginsd_root = NULL;
 
 static inline int pluginsd_space(char c) {
@@ -36,7 +34,7 @@ inline int config_isspace(char c) {
 }
 
 // split a text into words, respecting quotes
-inline int quoted_strings_splitter(char *str, char **words, int max_words, int (*custom_isspace)(char)) {
+static inline int quoted_strings_splitter(char *str, char **words, int max_words, int (*custom_isspace)(char)) {
     char *s = str, quote = 0;
     int i = 0, j;
 
@@ -101,6 +99,20 @@ inline int quoted_strings_splitter(char *str, char **words, int max_words, int (
     return i;
 }
 
+inline int pluginsd_initialize_plugin_directories() {
+    char plugins_dirs[(FILENAME_MAX * 2) + 1];
+    static char *plugins_dir_list = NULL;
+
+    // Get the configuration entry
+    if(likely(!plugins_dir_list)) {
+        snprintfz(plugins_dirs, FILENAME_MAX * 2, "\"%s\" \"%s/custom-plugins.d\"", PLUGINS_DIR, CONFIG_DIR);
+        plugins_dir_list = strdupz(config_get(CONFIG_SECTION_GLOBAL, "plugins directory",  plugins_dirs));
+    }
+
+    // Parse it and store it to plugin directories
+    return quoted_strings_splitter(plugins_dir_list, plugin_directories, PLUGINSD_MAX_DIRECTORIES, config_isspace);
+}
+
 inline int pluginsd_split_words(char *str, char **words, int max_words) {
     return quoted_strings_splitter(str, words, max_words, pluginsd_space);
 }
@@ -142,7 +154,12 @@ inline size_t pluginsd_process(RRDHOST *host, struct plugind *cd, FILE *fp, int
 
         char *r = fgets(line, PLUGINSD_LINE_MAX, fp);
         if(unlikely(!r)) {
-            error("read failed");
+            if(feof(fp))
+                error("read failed: end of file");
+            else if(ferror(fp))
+                error("read failed: input error");
+            else
+                error("read failed: unknown error");
             break;
         }
 
