@@ -14,6 +14,7 @@
 #define config_get_float(section, name, value) appconfig_get_float(&netdata_config, section, name, value)
 #define config_get_boolean(section, name, value) appconfig_get_boolean(&netdata_config, section, name, value)
 #define config_get_boolean_ondemand(section, name, value) appconfig_get_boolean_ondemand(&netdata_config, section, name, value)
+#define config_get_duration(section, name, value) appconfig_get_duration(&netdata_config, section, name, value)
 
 #define config_set(section, name, default_value) appconfig_set(&netdata_config, section, name, default_value)
 #define config_set_default(section, name, value) appconfig_set_default(&netdata_config, section, name, value)
@@ -70,8 +71,7 @@ extern char *netdata_configured_hostname;
 extern char *netdata_configured_user_config_dir;
 extern char *netdata_configured_stock_config_dir;
 extern char *netdata_configured_log_dir;
-extern char *netdata_configured_plugins_dir_base;
-extern char *netdata_configured_plugins_dir;
+extern char *netdata_configured_primary_plugins_dir;
 extern char *netdata_configured_web_dir;
 extern char *netdata_configured_cache_dir;
 extern char *netdata_configured_varlib_dir;
@@ -80,4 +80,6 @@ extern char *netdata_configured_host_prefix;
 extern char *netdata_configured_timezone;
 extern int netdata_anonymous_statistics_enabled;
 
+int netdata_ready;
+
 #endif /* NETDATA_COMMON_H */
