@@ -20,6 +20,18 @@ error() {
 	echo >&3 "$(date) : ERROR: " "${@}"
 }
 
+safe_sha256sum() {
+	# Within the contexct of the installer, we only use -c option that is common between the two commands
+	# We will have to reconsider if we start non-common options
+	if command -v sha256sum >/dev/null 2>&1; then
+		sha256sum $@
+	elif command -v shasum >/dev/null 2>&1; then
+		shasum -a 256 $@
+	else
+		fatal "I could not find a suitable checksum binary to use"
+	fi
+}
+
 # this is what we will do if it fails (head-less only)
 fatal() {
 	error "FAILED TO UPDATE NETDATA : ${1}"
@@ -55,7 +67,13 @@ download() {
 }
 
 set_tarball_urls() {
-	if [ "$1" == "stable" ]; then
+
+	if [ ! -z "${NETDATA_LOCAL_TARBAL_OVERRIDE}" ]; then
+		info "Not fetching remote tarballs, local override was given"
+		return
+	fi
+
+	if [ "$1" = "stable" ]; then
 		local latest
 		# Simple version
 		# latest="$(curl -sSL https://api.github.com/repos/netdata/netdata/releases/latest | grep tag_name | cut -d'"' -f4)"
@@ -71,46 +89,57 @@ set_tarball_urls() {
 update() {
 	[ -z "${logfile}" ] && info "Running on a terminal - (this script also supports running headless from crontab)"
 
-	dir=$(create_tmp_directory)
-	cd "$dir"
+	RUN_INSTALLER=0
+	tmpdir=$(create_tmp_directory)
+	cd "$tmpdir"
 
-	download "${NETDATA_TARBALL_CHECKSUM_URL}" "${dir}/sha256sum.txt" >&3 2>&3
-	if grep "${NETDATA_TARBALL_CHECKSUM}" sha256sum.txt >&3 2>&3; then
-		info "Newest version is already installed"
-		exit 0
-	fi
-
-	download "${NETDATA_TARBALL_URL}" "${dir}/netdata-latest.tar.gz"
-	if ! grep netdata-latest.tar.gz sha256sum.txt | sha256sum --check - >&3 2>&3; then
-		failed "Tarball checksum validation failed. Stopping netdata upgrade and leaving tarball in ${dir}"
-	fi
-	NEW_CHECKSUM="$(sha256sum netdata-latest.tar.gz 2>/dev/null| cut -d' ' -f1)"
-	tar -xf netdata-latest.tar.gz >&3 2>&3
-	rm netdata-latest.tar.gz >&3 2>&3
-	cd netdata-*
-
-	# signal netdata to start saving its database
-	# this is handy if your database is big
-	pids=$(pidof netdata)
-	do_not_start=
-	if [ -n "${pids}" ]; then
-		#shellcheck disable=SC2086
-		kill -USR1 ${pids}
+	if [ -z "${NETDATA_LOCAL_TARBAL_OVERRIDE}" ]; then
+		download "${NETDATA_TARBALL_CHECKSUM_URL}" "${tmpdir}/sha256sum.txt" >&3 2>&3
+		if [[ -n "${NETDATA_TARBALL_CHECKSUM}" ]] && grep "${NETDATA_TARBALL_CHECKSUM}" sha256sum.txt >&3 2>&3; then
+			info "Newest version is already installed"
+		else
+			download "${NETDATA_TARBALL_URL}" "${tmpdir}/netdata-latest.tar.gz"
+			if ! grep netdata-latest.tar.gz sha256sum.txt | safe_sha256sum -c - >&3 2>&3; then
+				fatal "Tarball checksum validation failed. Stopping netdata upgrade and leaving tarball in ${tmpdir}"
+			fi
+			NEW_CHECKSUM="$(safe_sha256sum netdata-latest.tar.gz 2>/dev/null| cut -d' ' -f1)"
+			tar -xf netdata-latest.tar.gz >&3 2>&3
+			rm netdata-latest.tar.gz >&3 2>&3
+			cd netdata-*
+			RUN_INSTALLER=1
+		fi
 	else
-		# netdata is currently not running, so do not start it after updating
-		do_not_start="--dont-start-it"
+		info "!!Local tarball override detected!! - Entering directory ${NETDATA_LOCAL_TARBAL_OVERRIDE} for installation, not downloading anything"
+		RUN_INSTALLER=1
+		cd ${NETDATA_LOCAL_TARBAL_OVERRIDE}
 	fi
 
-	info "Re-installing netdata..."
-	eval "${REINSTALL_COMMAND} --dont-wait ${do_not_start}" >&3 2>&3 || fatal "FAILED TO COMPILE/INSTALL NETDATA"
-	sed -i '/NETDATA_TARBALL/d' "${ENVIRONMENT_FILE}"
-	cat <<EOF >>"${ENVIRONMENT_FILE}"
+
+	# We got the sources, run the update now
+	if [ ${RUN_INSTALLER} -eq 1 ]; then
+		# signal netdata to start saving its database
+		# this is handy if your database is big
+		pids=$(pidof netdata)
+		do_not_start=
+		if [ -n "${pids}" ]; then
+			#shellcheck disable=SC2086
+			kill -USR1 ${pids}
+		else
+			# netdata is currently not running, so do not start it after updating
+			do_not_start="--dont-start-it"
+		fi
+
+		info "Re-installing netdata..."
+		eval "${REINSTALL_COMMAND} --dont-wait ${do_not_start}" >&3 2>&3 || fatal "FAILED TO COMPILE/INSTALL NETDATA"
+		sed -i '/NETDATA_TARBALL/d' "${ENVIRONMENT_FILE}"
+		cat <<EOF >>"${ENVIRONMENT_FILE}"
 NETDATA_TARBALL_CHECKSUM="$NEW_CHECKSUM"
 EOF
+	fi
 
-	rm -rf "${dir}" >&3 2>&3
+	rm -rf "${tmpdir}" >&3 2>&3
 	[ -n "${logfile}" ] && rm "${logfile}" && logfile=
-	return 0
+	return
 }
 
 # Usually stored in /etc/netdata/.environment
