@@ -129,6 +129,7 @@ struct web_client {
 
     char decoded_url[NETDATA_WEB_REQUEST_URL_SIZE + 1];  // we decode the URL in this buffer
     char last_url[NETDATA_WEB_REQUEST_URL_SIZE+1];       // we keep a copy of the decoded URL here
+    char host[256];
 
     struct timeval tv_in, tv_ready;
 
@@ -153,6 +154,9 @@ struct web_client {
     // STATIC-THREADED WEB SERVER MEMBERS
     size_t pollinfo_slot;           // POLLINFO slot of the web client
     size_t pollinfo_filecopy_slot;  // POLLINFO slot of the file read
+#ifdef ENABLE_HTTPS
+    struct netdata_ssl ssl;
+#endif
 };
 
 extern uid_t web_files_uid(void);
